// read data from dict and calculate general values

Info << "Reading data" << endl;

fvMesh mesh
(
    IOobject
    (
        fvMesh::defaultRegion,
        runTime.timeName(),
        runTime,
        Foam::IOobject::MUST_READ
    )
);

IOdictionary udvDict
(
    IOobject
    (
        "udv",
        runTime.constant(),
        mesh,
        IOobject::MUST_READ_IF_MODIFIED,
        IOobject::NO_WRITE
    )
);

const dimensionedScalar pi("pi", dimensionSet(0,0,0,0,0,0,0), 3.14159265359);

const dimensionedScalar D("D", dimensionSet(0,1,0,0,0,0,0), udvDict);      // piezo diameter
const dimensionedScalar f("f", dimensionSet(0,0,-1,0,0,0,0), udvDict);     // piezo frequency
const dimensionedScalar c("c", dimensionSet(0,1,-1,0,0,0,0), udvDict);     // sound velocity
const vector p(udvDict.lookup("startPoint"));       // location of probe
const vector p2(udvDict.lookup("endPoint"));	    // ond point of sampling-line
int n = udvDict.lookupOrDefault<int>("n",10); 	    // number of sample points
n--;						    // exclude the first point, because 0

const dimensionedScalar lambda = c / f;             		// wave length
const dimensionedScalar N = (D*D - lambda*lambda)/(4*lambda);	// near field length
vector dir = (p2 - p)/Foam::mag(p2 - p);                       	// direction of beam
dimensionedScalar theta_cr  = Foam::asin(1.22*lambda/D);        // half angle of the beam; value 1.22 from Krautkraemer p.73


dimensionedScalar thetaBeam = Foam::acos(dir & vector(0,0,1));	// direction of beam in spherical coordinates
dimensionedScalar phiBeam   = Foam::atan2(dir & vector(0,1,0), dir & vector(1,0,0));

Info << "    Wavelength: " << lambda.value()*1000. << " mm" << endl;
Info << "    near field: " << N.value()*1000. << " mm" << endl;
Info << "    beam vector: " << dir << endl;
Info << "    half angle of beam: " << (180*theta_cr/pi).value() << "°" << endl;
Info << "    direction of beam: phi=" << (180*phiBeam/pi).value() << "°; theta=" << (180*thetaBeam/pi).value() << "°" << endl;


// if specified, sample points will be written in the files p*
bool writeCloud = false;
if(args.found("writeCloud"))
{
    writeCloud = true;
}
PtrList<OFstream> file(n);
OFstream allPoints("p.csv");		// all interpolation files will be written into this file
allPoints << "x, y, z" << endl;

if(writeCloud)
{
    for(int i=0;i<n;i++)
    {
	std::stringstream ss;
   	ss << i;

	file.set(i, new OFstream(("p" + ss.str()+".csv")));
    }
}
